package net.wanji.web.task;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.common.constant.Constant;
import net.wanji.web.common.util.StringUtils;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.mapper.TDeviceStatusMapper;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.vo.situationDetection.CommonEventAlarmOutVo;
import net.wanji.web.websocket.WebSocketServer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/06 10:45
 */
@Component
@Slf4j
public class  CommonEventWarnTask {

    @Autowired
    private UtcFeignClients utcFeignClients;

    @Autowired
    private SituationDetectionService situationDetectionService;

    @Autowired
    private TDeviceStatusMapper tDeviceStatusMapper;

    @Scheduled(fixedRate = 30 * 1000)
    public void pushEvenWarnData() {
        try {
            Set<WebSocketServer> events = WebSocketServer.getWebSocketSet(Constant.WEBSOCKET_TOPIC_EVENT_WARN);
            if (!CollectionUtils.isEmpty(events)) {
                for (WebSocketServer entry : events) {
                    String userId = entry.getUserId();
                    getCommonEnventWarnData(userId);
                }
            }
        } catch (Exception e) {
            log.error("统一推送事件告警数据失败！", e);
        }
    }

    private void getCommonEnventWarnData(String userId) throws IOException {
        // 信号机故障
        JsonViewObject signalWarnData = getSignalWarnData();
        if (Objects.nonNull(signalWarnData)) {
            WebSocketServer.sendInfo(JSONObject.toJSONString(signalWarnData), Constant.WEBSOCKET_TOPIC_EVENT_WARN, userId);
        }
        // todo 数据中心获取路口告警
        JsonViewObject crossWarnData = getCrossWarnData();
        // todo 数据中心获取事件告警
        JsonViewObject crossEventWarnData = getCrossEventWarnData();
    }

    private JsonViewObject getCrossEventWarnData() {
        return null;
    }

    private JsonViewObject getCrossWarnData() {
        return null;
    }

    /**
     * 信号机故障告警数据
     *
     * @return
     */
    private JsonViewObject getSignalWarnData() {
        List<CommonEventAlarmOutVo> results = new ArrayList<>();
        JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();

        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机告警UTC服务调用异常！");
        }
        LocalDateTime now = LocalDateTime.now();
        String formatNow = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(now);
        List<Map<String, Object>> content = (List<Map<String, Object>>) jsonViewObject.getContent();
        content.forEach(map -> {
            Map<String, Object> result = map;
            String signalId = (String) result.get("signalId");
            String crossId = (String) result.get("crossId");
            Integer currentStatus = (Integer) result.get("status");
            Integer currentFaultType = (Integer) result.get("faultType");
            String coordinate = "";
            String crossName = "";
            int faultTypeDB = 0;
            int statusDB = 0;
            if (StringUtils.isNotEmpty(crossId)) {
                CrossInfoPO tBaseCrossInfo = situationDetectionService.selectCrossInfoById(crossId);
                if (Objects.nonNull(tBaseCrossInfo)) {
                    coordinate = tBaseCrossInfo.getLocation();
                }
                QueryWrapper<TDeviceStatusInfo> queryWrapper = new QueryWrapper<>();
                TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(queryWrapper.eq("code", crossId));
                if (Objects.nonNull(tDeviceStatusInfo)) {
                    faultTypeDB = tDeviceStatusInfo.getFaultType();
                    statusDB = tDeviceStatusInfo.getStatus() == 2 ? 1 : tDeviceStatusInfo.getStatus();
                    crossName = tDeviceStatusInfo.getName();
                }
            }
            if ((Objects.nonNull(currentFaultType) && currentFaultType != faultTypeDB) || currentStatus != statusDB) {
                CommonEventAlarmOutVo commonEventAlarmOutVo = new CommonEventAlarmOutVo();
                commonEventAlarmOutVo.setEventType(3);
                commonEventAlarmOutVo.setEventId(signalId);
                commonEventAlarmOutVo.setEventName(crossName);
                if (currentStatus == 0) {
                    commonEventAlarmOutVo.setStatus("11");
                    commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机离线"));
                } else if (currentStatus == 1){
                    commonEventAlarmOutVo.setStatus(String.valueOf(0));
                    commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机上线"));
                } else {
                    commonEventAlarmOutVo.setStatus(String.valueOf(currentFaultType));
                    commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机故障"));
                }
                commonEventAlarmOutVo.setStartTime(formatNow);
                commonEventAlarmOutVo.setCoordinate(coordinate);
                results.add(commonEventAlarmOutVo);
            }
        });
        // 无告警数据
        if (CollectionUtils.isEmpty(results)) {
            return null;
        }
        jsonViewObject.setTimestamp(now.toInstant(ZoneOffset.of("+8")).toEpochMilli());
        jsonViewObject.setContent(results);
        return jsonViewObject;
    }
}
