package net.wanji.utc.task;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.vo.LightsStatusVO;
import net.wanji.utc.websocket.RealTimeDataWebSocket;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 订阅信号运行状态、告警、灯态
 *
 * @author Kent HAN
 * @date 2022/11/21 9:01
 */
@Component
@Slf4j
@SuppressWarnings("all")
public class SignalStatusTask {
    @Autowired
    SignalStatusService signalStatusService;
    @Resource(name = "threadPoolExecutor")
    ThreadPoolTaskExecutor threadPoolExecutor;

    @Resource
    private HkLightsStatusService hkLightsStatusService;

    private static Map<String, List<LightsStatusVO>> lightStatusCache = new HashMap<>();
    private static final Map<String, Integer> runModeTimeMap = new ConcurrentHashMap<>();

    // 运行状态、告警，5分钟一次
    @Scheduled(fixedRate = 1 * 60 * 1000)
    public void runningStatusAlarm() {
        List<SignalStatusLogPO> signalStatusLogPOS = signalStatusService.runningStatusAlarm();
    }

    // 灯态，每秒一次
    @Scheduled(fixedRate = 1 * 1000)
    public void lightStatus() {

        ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap = RealTimeDataWebSocket.getEvenWarnWebSocketMap();
        try {
            if (!evenWarnWebSocketMap.isEmpty()) {
                for (Map.Entry<String, RealTimeDataWebSocket> entry : evenWarnWebSocketMap.entrySet()) {
                    threadPoolExecutor.execute(() -> {
                        String crossId = null;
                        String crossIdStr = entry.getKey();
                        if (StringUtils.isNotBlank(crossIdStr)) {
                            String[] split = crossIdStr.split(",");
                            crossId = split[0];
                        }

                        long start = System.currentTimeMillis();
                        List<LightsStatusVO> lightsStatusVOS = hkLightsStatusService.getHkLightsStatus(crossId);
                        long end = System.currentTimeMillis();
                        log.info("当前路口编号：{}，海康返回灯态时间：{}ms", crossId, (end - start));

                        RealTimeDataWebSocket webSocket = entry.getValue();
                        if (!CollectionUtils.isEmpty(lightsStatusVOS)) {
                            for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
                                if (Objects.equals(lightsStatusVO.getCrossId(), crossIdStr.split(",")[0])) {
                                    Integer coutTime = 0;
                                    if (Arrays.asList(2, 3, 4, 9).contains(Integer.valueOf(lightsStatusVO.getRunMode()))) {
                                        if (runModeTimeMap.isEmpty() || Objects.isNull(runModeTimeMap.get(crossIdStr))) {
                                            runModeTimeMap.put(crossIdStr, coutTime);
                                        } else {
                                            Integer countTimeCahce = runModeTimeMap.get(crossIdStr);
                                            runModeTimeMap.put(crossIdStr, ++countTimeCahce);
                                        }
                                        lightsStatusVO.setCycleCountDown(runModeTimeMap.get(crossIdStr));
                                        log.info("路口编号：{}，控制剩余时长：{}", crossIdStr, runModeTimeMap.get(crossIdStr));
                                    } else if (StringUtils.equals(lightsStatusVO.getRunMode(), "11")){
                                        runModeTimeMap.remove(crossIdStr);
                                    }
                                    try {
                                        webSocket.sendInfo(JSONObject.toJSONString(lightsStatusVOS, SerializerFeature.WriteMapNullValue), crossIdStr);
                                    } catch (Exception e) {
                                        log.error("RealTimeDataWebSocket发送异常，异常信息：", e);
                                    }
                                }
                            }
                        }
                    });
                }
            }
        } catch (Exception e) {
            log.error("实时推送灯态数据失败", e);
        }
    }
}
